from typing import Any

class RequestMethods:
    headers: Any
    def __init__(self, headers=None) -> None: ...
    def urlopen(self, method, url, body=None, headers=None, encode_multipart=True, multipart_boundary=None, **kw): ...
    def request(self, method, url, fields=None, headers=None, **urlopen_kw): ...
    def request_encode_url(self, method, url, fields=None, **urlopen_kw): ...
    def request_encode_body(
        self, method, url, fields=None, headers=None, encode_multipart=True, multipart_boundary=None, **urlopen_kw
    ): ...
